@@ -71,9 +71,10 @@ use curve25519_dalek::scalar::Scalar;
 
 use hex_literal::hex;
 
-use crate::consensus::encode::{self, Decodable, Encodable};
+use crate::consensus::encode::{self, Decodable};
 use crate::cryptonote::hash;
 
+use sealed::sealed;
 use thiserror::Error;
 
 #[cfg(feature = "serde_support")]
@@ -110,7 +111,7 @@ impl PrivateKey {
     }
 
     /// Serialize the private key to bytes.
-    pub fn to_bytes(&self) -> [u8; 32] {
+    pub fn to_bytes(self) -> [u8; 32] {
         self.scalar.to_bytes()
     }
 
@@ -259,7 +260,8 @@ impl Decodable for PrivateKey {
     }
 }
 
-impl Encodable for PrivateKey {
+#[sealed]
+impl crate::consensus::encode::Encodable for PrivateKey {
     fn consensus_encode<S: io::Write>(&self, s: &mut S) -> Result<usize, io::Error> {
         self.to_bytes().consensus_encode(s)
     }
@@ -280,7 +282,7 @@ impl PublicKey {
     }
 
     /// Serialize a public key to bytes.
-    pub fn to_bytes(&self) -> [u8; 32] {
+    pub fn to_bytes(self) -> [u8; 32] {
         self.point.to_bytes()
     }
 
@@ -471,7 +473,8 @@ impl Decodable for PublicKey {
     }
 }
 
-impl Encodable for PublicKey {
+#[sealed]
+impl crate::consensus::encode::Encodable for PublicKey {
     fn consensus_encode<S: io::Write>(&self, s: &mut S) -> Result<usize, io::Error> {
         self.to_bytes().consensus_encode(s)
     }
@@ -550,11 +553,10 @@ mod tests {
 
     #[test]
     fn parse_public_key() {
-        assert_eq!(
-            true,
-            PublicKey::from_str("eac2cc96e0ae684388e3185d5277e51313bff98b9ad4a12dcd9205f20d37f1a3")
-                .is_ok()
-        );
+        assert!(PublicKey::from_str(
+            "eac2cc96e0ae684388e3185d5277e51313bff98b9ad4a12dcd9205f20d37f1a3"
+        )
+        .is_ok());
     }
 
     #[test]
